#! /usr/bin/perl
#-*- tab-width: 4; -*-
# ex:ts=4
#
# portlint - lint for port directory
#
# this version implemented by:
#   Michael Haro <mharo@FreeBSD.org>
# versions thru 2.x implemented by:
#	Jun-ichiro itojun Hagino <itojun@itojun.org>
#	Yoshishige Arai <ryo2@on.rim.or.jp>
#
# Copyright(c) 2000,2001 by Michael Haro <mharo@FreeBSD.org>.
# Copyright(c) 1997 by Jun-ichiro Hagino <itojun@itojun.org>.
# All rights reserved.
# Freely redistributable.  Absolutely no warranty.
#
# $FreeBSD: ports/devel/portlint/src/portlint.pl,v 1.15 2000/03/20 18:03:01 mharo Exp $
# $Id: portlint.pl,v 1.45 2001/03/30 03:35:50 mharo Exp $
#

# My current idea is to have a bunch of really small
# functions that each do one (1) check and have main
# call all these functions on whatever files they operate on

#
# creating a new portlint check
# -----------------------------
# - create a seperate function to do the check in
# - the function should call &verbose to report no errors
#
# the reason we use seperate functions for everything is to make it 
# easy to disable indivisual checks with a command line option
# (there is a reason we are using Getopt::Long)
#
# creating a new set of functions (seperate file)
# -------------------------------
# return an array of the following hash structure (one element per function)
# {
#   function => \&function # pointer to function
#   name => "Name",        # English name for check
#   option => "option"     # Commmand line options which enable/disable check",
#   enabled => 1,    # 1 = enabled, 0 = disabled
#   file => "Makefile",    # file to check
# }

use strict;
use Getopt::Long;

########################################
# All global variables defined here
########################################

# my name
my ($prog) = ($0 =~ /([^\/]+)$/);

# version variables
my $major = 3;
my $minor = 0;
my $patchlevel = 0;
my $version = "$major.$minor.$patchlevel";

# Command line options hash
my %opts;

# number of warnings and fatal errors
my $nwarn = 0;
my $nfatal = 0;

my $patchcount = 0;
my %checkedurls;

my @func_reg = ();

######################################## LOAD SCRIPTS

# Most portlint checks before the breakup
require "/home/mharo/work/src/portlint/share/portlint/portlint-new.pl";

# Generic functions
require "/home/mharo/work/src/portlint/share/portlint/generic.pl";

# Makefile checks
push(@func_reg, @{require "/home/mharo/work/src/portlint/share/portlint/makefile.pl"});
push(@func_reg, @{require "/home/mharo/work/src/portlint/share/portlint/makefilesections.pl"});

# pkg-plist checks
push(@func_reg, @{require "/home/mharo/work/src/portlint/share/portlint/plistchecks.pl"});

require "/home/mharo/work/src/portlint/share/portlint/makefile2.pl";

######################################## START LINTING

&main;

errx(1, "got to end of main, but didn't exit");

######################################## OTHER STUFF

# put main into a subroutine to take care of random global vars
sub main {
	my @varnames = ();
	my @sectvars = ();

	Getopt::Long::Configure("no_auto_abbrev", "no_getopt_compat",
							"bundling", "ignore_case");
	my $result = GetOptions(\%opts,
							"b",
							"checkall|a",
							"contblank|B=i",
							"committer|c",
							"patch",
							"patchfile=s",
							"display-makevars!",
							"display-options!",
							"help|h|H",
							"newport|N",
							"make_vars|M",
							"tabs|t",
							"warnings-fatal",
							"version|V",
							"verbose|v",
							);

	&usage if $result eq '' || $opts{help};
	&version if ($opts{version});

	my $portdir = $ARGV[0] ? $ARGV[0] : ".";
	&verbose("linting directory $portdir");
	chdir($portdir) || &err(1, "chdir $portdir");

# display-options if verbose and nodisplay-options is not defined
# or if display-options is defined
	if ((defined($opts{'display-options'}) && $opts{'display-options'} != 0 && $opts{'verbose'}) || ($opts{'verbose'} && !defined($opts{'display-options'})) || $opts{'display-options'} ) {
		foreach my $var (sort keys %opts) {
			print "option: $var = $opts{$var}\n";
		}
	}

# init stuff
	my $contblank = $opts{contblank} ? $opts{contblank} : 1;

my @varlist = (split(/\s+/, <<EOF));
PORTNAME PORTVERSION PORTREVISION PORTEPOCH PKGNAME PKGNAMEPREFIX
PKGNAMESUFFIX DISTNAME DISTFILES CATEGORIES MASTERDIR MAINTAINER
MASTER_SITES WRKDIR WRKSRC NO_WRKSUBDIR PATCHDIR SCRIPTDIR FILESDIR PKGDIR
COMMENT DESCR PLIST MD5_FILE .CURDIR VALID_CATEGORIES NO_CHECKSUM
MAN1 MAN2 MAN3 MAN4 MAN5 MAN6 MAN7 MAN8 MAN9 MANL MANN
EOF
	my %makevar = &getmakevars(@varlist);

@varlist = (split(/\s+/, <<EOF));
MASTER_SITE_AFTERSTEP MASTER_SITE_COMP_SOURCES MASTER_SITE_GNOME
MASTER_SITE_GNU MASTER_SITE_KDE MASTER_SITE_PERL_CPAN MASTER_SITE_PORTS_JP
MASTER_SITE_RUBY MASTER_SITE_SOURCEFORGE MASTER_SITE_SUNSITE MASTER_SITE_TCLTK
MASTER_SITE_TEX_CTAN MASTER_SITE_THEMES MASTER_SITE_WINDOWMAKER
MASTER_SITE_XCONTRIB MASTER_SITE_XFREE
EOF
	my %mastersites = &getmakevars(@varlist);

	my $pwd = $makevar{".CURDIR"};

# remove these in the future
	my $file;
	my ($whole, $linecnt, $linelen);

# Makefile checks
	$file = "Makefile";
	&verbose("checking $file");
	($whole, $linecnt, $linelen) = &loadfile($file);

	&functionloop($file, $whole);

	&whitespaceatend($file, $whole);
	&blanklines($file, $whole, $contblank);
	&lastline($file, $whole);
	&nouservar($file, "NO_CHECKSUM", $makevar{NO_CHECKSUM});
	# any other vars that users define?
	&noportdocs($file, $whole, $makevar{PREFIX});

	# direct use of command names check
	# full paths check
	
	# check for duplicate var defs
	# check that MANX= foo.Y  has X == Y
	# check for duplicate manpage listings in MANX
	&propermansect(%makevar);
	# check for MASTER_SITES which are defined in bsd.port.mk

#MICHAEL:  add something to deal with master/slave/independent ports


# pkg-descr checks
	$file = "pkg-descr";
	$file = $makevar{DESCR} if $makevar{DESCR} ne "";
	$file =~ s,^$pwd/,,;
	&verbose("checking $file");
	($whole, $linecnt, $linelen) = &loadfile($file);

	&functionloop($file, $whole);

	&maxlines($file, $linecnt, 24);
	&maxchars($file, $linelen, 80);
	&descrwwwcheck($file, $whole);
	&whitespaceatend($file, $whole);
	&blanklines($file, $whole, $contblank);
	&lastline($file, $whole);

# pkg-comment checks
	$file = "pkg-comment";
	$file = $makevar{COMMENT} if $makevar{COMMENT} ne "";
	$file =~ s,^$pwd/,,;
	&verbose("checking $file");
	($whole, $linecnt, $linelen) = &loadfile($file);

	&functionloop($file, $whole);

	&maxlines($file, $linecnt, 1);
	&maxchars($file, $linelen, 70);
	&whitespaceatend($file, $whole);
	&lastline($file, $whole);

# pkg-plist checks
	$file = "pkg-plist";
	$file = $makevar{PLIST} if $makevar{PLIST} ne "";
	$file =~ s,^$pwd/,,;
	&verbose("checking $file");
	($whole, $linecnt, $linelen) = &loadfile($file);

	&functionloop($file, $whole);

	&blanklines($file, $whole, 0);
	&whitespaceatend($file, $whole);
	&lastline($file, $whole);
	# plist @exec/@unexec install-info ordering check
	# deal with multiple PLIST files

# distinfo checks
	$file = "distinfo";
	$file = $makevar{MD5_FILE} if $makevar{MD5_FILE} ne "";
	$file =~ s,^$pwd/,,;
	&verbose("checking $file");
	($whole, $linecnt, $linelen) = &loadfile($file);

	&functionloop($file, $whole);


# files/ related checks
	# check that there is only 1 file being patched per patch file
	# check that there is only 1 patch file for each file being patched
	# &lastline foreach file in patches/

# scripts/ related checks
	# &lastline foreach file in scripts/

# other checks
	&workdircheck;
	&warncrud(".");

# We are done.  Display number of warnings and errors
	if ($nfatal || $nwarn) {
		print "$nfatal fatal errors and $nwarn warnings found.\n";
	} else {
		print "looks fine.\n";
	}
	
	exit $nfatal;
}

# eventually usage should be copied into the portlint manpage
# so for now this might be a little too verbose
sub usage {
	print STDERR <<EOF;
usage: $prog [options] [port_directory]
    -B#, --contblank#  allow # contiguous blank lines (default: 1 line)
    -b  warn \$(VARIABLE)
    -t, --tabs         nit pick about use of spaces.
    -v, --verbose      verbose mode
    --warnings-fatal   treat warnings as fatals.
    --patch            create a portlint generated patchfile with suggested
                       changes which can be applied to the port.
    --patchfile=name   change the name of the portlint generated patch file.

put something here about how the following options can be disabled with 'no'
    --display-makevars display variables taken from make.
                       defaults to on when in verbose mode.  May
                       be disabled by --nodisplay-options
    --display-options  show which command line options were seen.
                       defaults to on when in verbose mode.  May
                       be disabled by --nodisplay-options


un-implemented portlint 2.2.2 flags:
    -a  additional check for scripts/* and pkg/*
    -c  committer mode
    -M  set make variables (ex. PORTSDIR=/usr/ports.work)
    -N  writing a new port
EOF
		exit 0;
}

sub version {
	print "$prog version $version\n";
	exit 0;
}

sub functionloop {
	my ($file, $whole) = @_;

	foreach my $item (@func_reg) {
		if ($item->{file} eq "$file" || $item->{file} eq undef) {
			if ($item->{enabled}) {  ## add command line options stuff here
				print "running check $item->{name} on file $file\n";
				&{$item->{function}}($file, $whole);
			}
		}
	}
}

######################################## OLD PORTLINT CHECKS

print "****************************************\n";

require "/home/mharo/work/src/portlint/share/portlint/portlint-old.pl";
